#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Conversión del indicador nonlagma del metatrader
    /// </summary>
    [Description("Conversión del indicador nonlagma del metatrader")]
    public class ZiNonLagma : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int periodo = 21; // Default setting for periodo
		private double filter = 3;
		private double splitTick = 1;
		private int cycle = 4;		
		private double coeff = 3;
		private double shift = 0.0;
        // User defined variables (add any user defined variables below)
		private DataSeries serie;
//		private DataSeries trend;
		private DataSeries sTrend;
		private int phase = 0;
		private double len = 0;		
		//private double deviation = 0.0;
		private double calc_Coeff = 0.0;
		private bool bInit = true;		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {	
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "Up Trend"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "Down Trend"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "Non Trend"));
			
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			
			serie = new DataSeries(this);
			sTrend = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {            
			if ( bInit )
			{
				calc_Coeff = coeff * Math.PI;
				phase = periodo - 1;
				len = periodo * cycle + phase;			
				bInit = false;
			}
			
			
			double g, t, beta, alfa, Sum, Weight;
			t = 0; Sum = 0; Weight = 0;												
			
			if ( CurrentBar < len ) return;

			
			for ( int i = 0; i <= len-1; i++ )
			{	
				if ( i <= phase-1) t = 1.0 * i/(phase-i);
				else t = 1.0 + (i-phase+1) * (2.0*cycle-1.0) / (cycle*periodo-1.0);
				
				beta = Math.Cos( Math.PI * t );
				g = 1 / ( calc_Coeff * t + 1 );				
				if ( t <= 0.5 ) g = 1;
				alfa = g * beta;
				Sum += alfa*Close[i];
				Weight += alfa;
			}
			
			serie[0] = serie[1];
			
			if ( Weight > 0 )
				serie.Set( 0, (1.0 + shift/100 ) * Sum/Weight );
			
			if ( Math.Abs( serie[0] - serie[1] ) < filter * TickSize / splitTick )
				serie.Set(0, serie[1]);
			
			// Plot Non Trend
			Values[2].Set( 0, serie[0] );
//			sTrend.Set(0, 0);
				
			sTrend[0] = sTrend[1];
			
			if ( serie[0] - serie[1] > filter * TickSize / splitTick )			
				sTrend.Set( 0, 1);				            
			else if ( serie[1] - serie[0] > filter * TickSize / splitTick ) 						
				sTrend.Set(0, -1);
			else 
				sTrend.Set(0, 0);
			
			
			if ( sTrend[0] > 0 && sTrend[1] > 0 )
			{
				// Plot Up Trend
				Values[0].Set(0, serie[0] );				
			}
			else if ( sTrend[0] < 0 && sTrend[1] < 0 )
			{
				// Plot Down Trend
				Values[1].Set(0, serie[0] );				
			}										
        }

        #region Properties        

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries UpTrend
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DownTrend
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries NonTrend
        {
            get { return Values[2]; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries STrend
        {
            get { Update(); return sTrend; }
        }		
		
        [Description("Período")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("1 Period")]
        public int Periodo
        {
            get { return periodo; }
            set { periodo = Math.Max(1, value); }
        }
		
		[Description("Ciclo")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("2 Cycle")]
        public int Cycle
        {
            get { return cycle; }
            set { cycle = Math.Max(1, value); }
        }
		
		[Description("Coeficiente")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("3 Coeff")]
        public double Coeff
        {
            get { return coeff; }
            set { coeff = Math.Max(1, value); }
        }
		
		[Description("Filtro")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("4 Filter")]
        public double Filter
        {
            get { return filter; }
            set { filter = Math.Max(0, value); }
        }
		
		[Description("Divisor del TickSize")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("5 SplitTick")]
        public double SplitTick
        {
            get { return splitTick; }
            set { splitTick = Math.Max(1, value); }
        }
		
		[Description("Desplazamiento")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("6 Shift")]
        public double  Shift
        {
            get { return shift; }
            set { shift = Math.Max(0, value); }
        }
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ZiNonLagma[] cacheZiNonLagma = null;

        private static ZiNonLagma checkZiNonLagma = new ZiNonLagma();

        /// <summary>
        /// Conversión del indicador nonlagma del metatrader
        /// </summary>
        /// <returns></returns>
        public ZiNonLagma ZiNonLagma(double coeff, int cycle, double filter, int periodo, double shift, double splitTick)
        {
            return ZiNonLagma(Input, coeff, cycle, filter, periodo, shift, splitTick);
        }

        /// <summary>
        /// Conversión del indicador nonlagma del metatrader
        /// </summary>
        /// <returns></returns>
        public ZiNonLagma ZiNonLagma(Data.IDataSeries input, double coeff, int cycle, double filter, int periodo, double shift, double splitTick)
        {
            checkZiNonLagma.Coeff = coeff;
            coeff = checkZiNonLagma.Coeff;
            checkZiNonLagma.Cycle = cycle;
            cycle = checkZiNonLagma.Cycle;
            checkZiNonLagma.Filter = filter;
            filter = checkZiNonLagma.Filter;
            checkZiNonLagma.Periodo = periodo;
            periodo = checkZiNonLagma.Periodo;
            checkZiNonLagma.Shift = shift;
            shift = checkZiNonLagma.Shift;
            checkZiNonLagma.SplitTick = splitTick;
            splitTick = checkZiNonLagma.SplitTick;

            if (cacheZiNonLagma != null)
                for (int idx = 0; idx < cacheZiNonLagma.Length; idx++)
                    if (Math.Abs(cacheZiNonLagma[idx].Coeff - coeff) <= double.Epsilon && cacheZiNonLagma[idx].Cycle == cycle && Math.Abs(cacheZiNonLagma[idx].Filter - filter) <= double.Epsilon && cacheZiNonLagma[idx].Periodo == periodo && Math.Abs(cacheZiNonLagma[idx].Shift - shift) <= double.Epsilon && Math.Abs(cacheZiNonLagma[idx].SplitTick - splitTick) <= double.Epsilon && cacheZiNonLagma[idx].EqualsInput(input))
                        return cacheZiNonLagma[idx];

            ZiNonLagma indicator = new ZiNonLagma();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Coeff = coeff;
            indicator.Cycle = cycle;
            indicator.Filter = filter;
            indicator.Periodo = periodo;
            indicator.Shift = shift;
            indicator.SplitTick = splitTick;
            indicator.SetUp();

            ZiNonLagma[] tmp = new ZiNonLagma[cacheZiNonLagma == null ? 1 : cacheZiNonLagma.Length + 1];
            if (cacheZiNonLagma != null)
                cacheZiNonLagma.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheZiNonLagma = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Conversión del indicador nonlagma del metatrader
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ZiNonLagma ZiNonLagma(double coeff, int cycle, double filter, int periodo, double shift, double splitTick)
        {
            return _indicator.ZiNonLagma(Input, coeff, cycle, filter, periodo, shift, splitTick);
        }

        /// <summary>
        /// Conversión del indicador nonlagma del metatrader
        /// </summary>
        /// <returns></returns>
        public Indicator.ZiNonLagma ZiNonLagma(Data.IDataSeries input, double coeff, int cycle, double filter, int periodo, double shift, double splitTick)
        {
            return _indicator.ZiNonLagma(input, coeff, cycle, filter, periodo, shift, splitTick);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Conversión del indicador nonlagma del metatrader
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ZiNonLagma ZiNonLagma(double coeff, int cycle, double filter, int periodo, double shift, double splitTick)
        {
            return _indicator.ZiNonLagma(Input, coeff, cycle, filter, periodo, shift, splitTick);
        }

        /// <summary>
        /// Conversión del indicador nonlagma del metatrader
        /// </summary>
        /// <returns></returns>
        public Indicator.ZiNonLagma ZiNonLagma(Data.IDataSeries input, double coeff, int cycle, double filter, int periodo, double shift, double splitTick)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ZiNonLagma(input, coeff, cycle, filter, periodo, shift, splitTick);
        }

    }
}
#endregion
